"use strict";

var Emitter = require("component-emitter"),
    Immutable = require("immutable");

var recorderOptions = {
  inProduction: false
};
var core = {
  inital: Immutable.Map(),
  updater: function updater(store) {
    console.warn("Default recorder updater is called.");
    return store;
  }
};

exports.setup = function (inital, updater, mode) {
  core = {
    inital: inital,
    records: Immutable.List(),
    updater: updater,
    pointer: -1
  };
  return recorderEmitter.inProduction = mode || false;
};

var recorderEmitter = new Emitter();

exports.subscribe = function (fn) {
  return recorderEmitter.on("update", fn);
};

exports.unsubscribe = function (fn) {
  return recorderEmitter.off("update", fn);
};

exports.dispatch = function (actionType, actionData) {
  if (recorderOptions.inProduction) {
    callUpdaterInProduction(actionType, actionData);
  } else {
    callUpdater(actionType, actionData);
  }

  return;
};

var callUpdaterInProduction = function callUpdaterInProduction(actionType, actionData) {
  actionData = Immutable.fromJS(actionData);
  var newStore = core.updater(core.inital, actionType, actionData);
  core.inital = newStore;
  core.records = core.records.push(Immutable.List([actionType, actionData]));
  return recorderEmitter.emit("update", newStore, core.records, core.pointer);
};

var callUpdater = function callUpdater(actionType, actionData) {
  actionData = Immutable.fromJS(actionData);
  var chunks = actionType.split("/");
  var groupName = chunks[0];

  if (groupName === "actions-recorder") {
    switch (chunks[1]) {
      case "commit":
        var newStore = core.records.reduce(function (acc, action) {
          return core.updater(acc, action.get(0), action.get(1));
        }, core.inital);
        core.inital = newStore;
        core.records = Immutable.List();
        core.pointer = -1;
        recorderEmitter.emit("update", newStore, core.records, core.pointer);
        break;

      case "reset":
        core.records = Immutable.List();
        core.pointer = -1;
        recorderEmitter.emit("update", core.inital, core.records, core.pointer);
        break;

      case "peek":
        core.pointer = actionData;
        var activeRecords = core.records.slice(0, core.pointer);
        var newStore = activeRecords.reduce(function (acc, action) {
          return core.updater(acc, action.get(0), action.get(1));
        }, core.inital);
        recorderEmitter.emit("update", newStore, core.records, core.pointer);
        break;

      case "discard":
        core.records = core.records.slice(0, core.pointer + 1);
        var newStore = core.records.reduce(function (acc, action) {
          return core.updater(acc, action.get(0), action.get(1));
        }, core.inital);
        recorderEmitter.emit("update", newStore, core.records, core.pointer);
        break;

      default:
        console.warn("Unknown actions-recorder action: " + actionType);
        break;
    }
  } else {
    core.records = core.records.push(Immutable.List([actionType, actionData]));
    var newStore = core.records.reduce(function (acc, action) {
      return core.updater(acc, action.get(0), action.get(1));
    }, core.inital);
    recorderEmitter.emit("update", newStore, core.records, core.pointer);
  }

  return undefined;
};